#!/bin/perl
# Unix Field Pack for EPAgent
# Copyright 2013 CA Technology, Inc. All rights reserved.
# Version 0.3 - 29 December 2006
# Sylvain Gibassier

# Note: This script creates a metric called "netstat:Success"
# The value for this metric is 2 is the script is functioning normally.
# If there is a problem, the value of this metric should be 1 and there
# should be a netstat_errors.log file in the logs directory. This file contains
# some diagnostic information for the problem that this script is experiencing.

# Fix details
# Author                 Date Modified       TT Details    Modification
# ------                 -------------       ----------    --------------------------------
# ghhsi01                15 May 2009         49952         Data format changed from
#                                                          XML format to simple data format 

use Sys::Hostname;
# warnings and strict commented out for the default version of Perl that comes with Solaris 8
# use warnings;
# use strict;
# Linux netstat script

my $error_log = "./../../logs/netstat_errors.log";

eval {
	my $host = hostname;
	$error_log = "./../../logs/netstat_errors.${host}.log";

	my $netstat_command = 'netstat -n -f inet|';

	if($^O eq 'linux') {
		$netstat_command = 'netstat -nt|';
	}

	open(NETSTAT,$netstat_command) || die "Error when executing \"$netstat_command\": $!";

	my $ESTABLISHED = 0;
	my $SYN_SENT = 0;
	my $SYN_RECV = 0;
	my $FIN_WAIT_1 = 0;
	my $FIN_WAIT_2 = 0;
	my $TIME_WAIT = 0;
	my $CLOSED = 0;
	my $CLOSE_WAIT = 0;
	my $LAST_ACK = 0;
	my $CLOSING = 0;
	my $UNKNOWN = 0;

	while (<NETSTAT>) {
		chomp;
		if (/ESTABLISHED/) {
			$ESTABLISHED++;
		} elsif (/SYN_SENT/) {
			$SYN_SENT++;
		} elsif (/SYN_RECV/) {
			$SYN_RECV++;
		} elsif (/FIN_WAIT_1/) {
			$FIN_WAIT_1++;
		} elsif (/FIN_WAIT_2/) {
			$FIN_WAIT_2++;
		} elsif (/TIME_WAIT/) {
			$TIME_WAIT++;
		} elsif (/CLOSED/) {
			$CLOSED++;
		} elsif (/CLOSE_WAIT/) {
			$CLOSE_WAIT++;
		} elsif (/LAST_ACK/) {
			$LAST_ACK++;
		} elsif (/CLOSING/) {
			$CLOSING++;
		} elsif (/UNKNOWN/) {
			$UNKNOWN++;
		} 
	}

	close(NETSTAT);

	my $total = $ESTABLISHED + $SYN_SENT + $SYN_RECV + $FIN_WAIT_1 + $FIN_WAIT_2 + $TIME_WAIT + $CLOSED + $CLOSE_WAIT + $LAST_ACK + $CLOSING + $UNKNOWN;

	print "OSResource-Linux|Network|TCP Connection States:Total ESTABLISHED=$ESTABLISHED\n";
	print "OSResource-Linux|Network|TCP Connection States:Total SYN_SENT=$SYN_SENT\n";
	print "OSResource-Linux|Network|TCP Connection States:Total SYN_RECV=$SYN_RECV\n";
	print "OSResource-Linux|Network|TCP Connection States:Total FIN_WAIT_1=$FIN_WAIT_1\n";
	print "OSResource-Linux|Network|TCP Connection States:Total FIN_WAIT_2=$FIN_WAIT_2\n";
	print "OSResource-Linux|Network|TCP Connection States:Total TIME_WAIT=$TIME_WAIT\n";
	print "OSResource-Linux|Network|TCP Connection States:Total CLOSED=$CLOSED\n";
	print "OSResource-Linux|Network|TCP Connection States:Total CLOSE_WAIT=$CLOSE_WAIT\n";
	print "OSResource-Linux|Network|TCP Connection States:Total LAST_ACK=$LAST_ACK\n";
	print "OSResource-Linux|Network|TCP Connection States:Total CLOSING=$CLOSING\n";
	print "OSResource-Linux|Network|TCP Connection States:Total UNKNOWN=$UNKNOWN\n";
	print "OSResource-Linux|Network:Total number of TCP connections=$total\n";
##### netstat cmd to display Network Interface details.
#The output is displayed under following header.
#Kernel Interface table
#Iface       MTU Met    RX-OK RX-ERR RX-DRP RX-OVR    TX-OK TX-ERR TX-DRP TX-OVR Flg
#eth0       1500   0    31786      0      0      0     3236      0      0      0 BMRU


	my $netstat_command1 = 'netstat -in|';
	open(NETSTAT,$netstat_command1) || die "Error when executing \"$netstat_command1\": $!";
	
	my $NetInterface = 0;
	my $InputPackets = 0;
	my $InputErrors = 0;
	my $InputDrop = 0;
	my $InputOverrun = 0;
	my $OutputPackets = 0;
	my $OutputErrors = 0;
	my $OutputDrop = 0;
	my $OutputOverrun = 0;
	my $TotalPackets = 0;
	my $TotalErrors = 0;
	my $TotalDrop = 0;
	my $TotalOverrun = 0;
	my $ignoreLine = 0;
	
	
	 while (<NETSTAT>) {
		chomp;
		split;
		if($ignoreLine < 2) {
			$ignoreLine++;
		 }
		 else{

	     $Count = $#_;
#		 print "Total count is $Count\n";
		 
		 $NetInterface = $_[0];
# 54648-fix
		if(!($NetInterface =~/:/))
		{
		 #Filed count should be 11. In case if there is no space delimiter between field "Met" and "RX-OK"
		 #field count is 10
		 if($Count < 11){
			 $Met = $_[2];
			 $InputPackets = substr($Met,1);
			 $InputErrors = $_[3];
			 $InputDrop = $_[4];
			 $InputOverrun = $_[5];
		 
			 $OutputPackets = $_[6];
			 $OutputErrors = $_[7];
			 $OutputDrop = $_[8];
			 $OutputOverrun = $_[9];
			
		 }
		 else
		 {		
		     $InputPackets = $_[3];
			 $InputErrors = $_[4];
			 $InputDrop = $_[5];
			 $InputOverrun = $_[6];
		 
			 $OutputPackets = $_[7];
			 $OutputErrors = $_[8];
			 $OutputDrop = $_[9];
			 $OutputOverrun = $_[10];
		 }
	     $InputPackets = int($InputPackets/1024);
	     $OutputPackets = int($OutputPackets/1024);
		 $TotalPackets = $InputPackets + $OutputPackets;
		 $TotalErrors = $InputErrors + $OutputErrors;
		 $TotalDrop = $InputDrop + $OutputDrop;
		 $TotalOverrun = $InputOverrun + $OutputOverrun;

		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Input Packets(MB)=$InputPackets\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Input Errors=$InputErrors\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Input Dropped=$InputDrop\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Input Lost=$InputOverrun\n";
		 
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Output Packets(MB)=$OutputPackets\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Output Errors=$OutputErrors\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Output Dropped=$OutputDrop\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Output Lost=$OutputOverrun\n";
		 
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Total Packets(MB)=$TotalPackets\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Total Packets Errors=$TotalErrors\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Total Dropped=$TotalDrop\n";
		 print "OSResource-Linux|Network|Network Interface|$NetInterface:Total Lost=$TotalOverrun\n";
		
		}		 
		}
	}
	close(NETSTAT);

#####

	#print qq{<metric type="IntCounter" name="OSResource|Network:Success" value="2"/>\n};
};

if ( $@ ) {
	# write the error message to an error log for troubleshooting
	open(ERROR_LOG,">>$error_log") || die "Error when opening file \"$error_log\": $!";
	print ERROR_LOG localtime() . " - " . $@;
	close(ERROR_LOG);

	# gives the end user the opportunity to monitor the vmstat plugin
	print qq{OSResource-Linux|Network:Success=1\n};	
}
