#!/bin/perl

# Copyright 2013 CA Technology, Inc. All rights reserved.
# Note: This script creates a metric called "OSResource|PolicyServerProcess|srvpolicysrv:Success"
# The value for this metric is 2 is the script is functioning normally.
# If there is a problem, the value of this metric should be 1 and there
# should be a process_errors.log file in the logs directory. This file contains
# some diagnostic information for the problem that this script is experiencing.

# Fix details
# Author                 Date Modified       TT Details    Modification
# ------                 -------------       ----------    --------------------------------
# ghhsi01                15 May 2009         49952         Data format changed from
#                                                          XML format to simple data format 

use Sys::Hostname;
# warnings and strict commented out for the default version of Perl that comes with Solaris 8
# use warnings;
# use strict;
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;


#Solaris
# ps -ef | grep -v grep | grep cam
#    root   325     1  0   Oct 24 ?        0:04 cam
#


#AIX
# ps -ef | grep -v grep | grep vmstat
#    root 237818 245826   0   May 02  pts/0  5:04 vmstat 7
#    root 245826 377052   0   May 02  pts/0  1:05 /usr/bin/perl epaplugins//aix//vmstat.pl


#HP
# ps -ef | grep -v grep | grep vmstat.pl
#    root   655   653  0  Feb  8  ?         0:07 /usr/bin/perl epaplugins//HP//vmstat.pl
#    root 24367 24348  0  Jan 25  pts/ta    1:19 /usr/bin/perl epaplugins//solaris//vmstat.pl
#    root 25489 25485  0  Feb  1  pts/tc    0:46 /usr/bin/perl epaplugins//HP//vmstat.pl


#linux
#ps -C smpolicysrv
# PID TTY          TIME CMD
#15731 ?        00:00:01 smpolicysrv



my $error_log = "./../../logs/process_errors.log";

eval {
	my $host = hostname;
	$error_log = "./../../logs/process_errors.${host}.log";


	my $pscommand = 'ps -ef | grep -v grep | grep smpolicysrv|';
        my $firstTime = 1;
	my $processName;
	my $smprocessName = 'smpolicysrv';
	my $processStatus= 0; # 0 for down and 1 for up
	
	
	if ($^O eq 'linux') {
		$pscommand = 'ps -C smpolicysrv|';
	} 


	open(PROCESS,$pscommand) || die "Error when executing \"$pscommand\": $!";

	 while (<PROCESS>) {
	
		if($^O eq 'linux' && $firstTime) {
			$firstTime = 0;
		}
		else{ 
				$processStatus = 1;
			}
		}
        	        Wily::PrintMetric::printMetric( type        => 'IntCounter',
                        	        	resource    => 'OSResource-Linux|PolicyServerProcess',
                                	      	subresource => $smprocessName,
                                      	      	name        => 'Status',
                                      	      	value       => $processStatus,
                                	    	);

	close(PROCESS);
};

if ( $@ ) {
	# write the error message to an error log for troubleshooting
	open(ERROR_LOG,">>$error_log") || die "Error when opening file \"$error_log\": $!";
	print ERROR_LOG localtime() . " - " . $@;
	close(ERROR_LOG);

	# gives the end user the opportunity to monitor the vmstat plugin
	print qq{OSResource-Linux|PolicyServerProcess|$processName:Success=1\n};	
}
